#include <avr/io.h>
//#include <util/delay.h>
#include <avr/interrupt.h>

const int OUTBYTES = 4;
typedef uint8_t combination[OUTBYTES];

const combination half1={ 0xB5, 0x1C, 0xAB, 0x75 };
const combination half2={ 0x4A, 0xE3, 0x54, 0x8A };
const combination halfletters = { 0xB5, 0x5A, 0x83, 0x7A };

const combination blaue = { 0x0, 0x0, 0x20, 0x0 };
const combination blauv = { 0x0, 0x0, 0x0, 0x4 };
const combination blaua = { 0x0, 0x4, 0x0, 0x0 };


const combination lettere = { 0xE0, 0xC0, 0xC0, 0xE0 };
const combination letterv = { 0x18, 0x38, 0x18, 0x18 };
const combination lettera = { 0x7, 0x3, 0x7, 0x3 };
const combination letter_tissue = { 0x0, 0x4, 0x20, 0x4 };
const combination allon = {0xff,0xff,0xff,0xff};
const combination alloff = {0x0,0x0,0x0,0x0};


volatile combination out;
volatile uint8_t dimmer = 0;

void load_combination(const combination in){
	for(uint8_t k = 0; k < OUTBYTES; ++k) out[k] = in[k];
}

void load_combination_or2(const combination in1, const combination in2){
	for(uint8_t k = 0; k < OUTBYTES; ++k) out[k] = in1[k] | in2[k];
}

void load_combination_or(const combination in2){
	for(uint8_t k = 0; k < OUTBYTES; ++k) out[k] |= in2[k];
}

void load_combination_neg(const combination in){
	for(uint8_t k = 0; k < OUTBYTES; ++k) out[k] = ~in[k];
}


inline void set_dimmer(uint8_t val){
	OCR2 = val;
}

ISR(TIMER1_COMPA_vect){
//	PORTD=~PORTD;
	TCNT1=0;

	static int counter = 0;
	++counter;
	if(counter < 12){
		if(counter % 2) load_combination(allon);
		else load_combination(alloff);
		switch(counter % 3){
			case 0: load_combination_or(blaue); break;
			case 1: load_combination_or(blauv); break;
			case 2: load_combination_or(blaua); break;
		}
	} else if(counter < 30){
		switch(counter % 3){
			case 0: load_combination(lettere); break;
			case 1: load_combination(letterv); break;
			case 2: load_combination(lettera); break;
			//case 3: load_combination(letter_tissue); break;
		}
	} else if(counter < 45) {
		switch(counter % 2){
			case 0: load_combination(halfletters); break;
			case 1: load_combination(alloff); break;
		}
	} else if(counter < 60){
		switch(counter % 3){
			case 0: load_combination_or2(lettere, letter_tissue); break;
			case 1: load_combination_or2(letterv, letter_tissue); break;
			case 2: load_combination_or2(lettera, letter_tissue); break;
			//case 3: load_combination(letter_tissue); break;
		}		
	} else if(counter == 60){
		load_combination(allon);
		set_dimmer(1);
	}
	else if(counter < 80) set_dimmer((254/20)*(counter - 60) +1);
	else if(counter == 80) set_dimmer(254);
	else if(counter < 100);
/*	} else if(counter < 100){
		switch(counter % 3){
			case 0: load_combination(blaue); break;
			case 1: load_combination(blauv); break;
			case 2: load_combination(blaua); break;
		}
	}
*/
	else if(counter < 120){
		switch(counter % 3){
			case 0: load_combination_neg(lettere); break;
			case 1: load_combination_neg(letterv); break;
			case 2: load_combination_neg(lettera); break;
			//case 3: load_combination(letter_tissue); break;
		}
	} else {
		counter = 0;
		OCR2 = 254;
	}

}

ISR(TIMER2_COMP_vect){
	PORTC = 0;
}

ISR(TIMER2_OVF_vect){
	static uint8_t counter = 0;
	if(++counter >= OUTBYTES) counter = 0;
	PORTD = out[counter];
	PORTC = (1 << (OUTBYTES - 1 - counter));
}

int main(){
	DDRD = 0xFF;
	DDRC = 0xF;
	PORTD = 0xFF;
	PORTC=0xF;

	OCR1A = (F_OSC / 1024);
	OCR2 = 254;
	TCCR1A = (1 << WGM12);			//clear on compare
	TCCR1B = (1 << CS12) | (1 << CS10);	//prescaler 1024
	//TCCR2 = (1 << CS22) | (1 << CS21) | (1 << CS20);
	TCCR2 = (1 << CS20);
	TIMSK = (1 << OCIE1A) | (1 << OCIE2) | (1 << TOIE2); 	//enable comp a int
//	TIMSK = (1 << OCIE1A); 	//enable comp a int
	sei();

	while(true){asm volatile("nop");	}

}
